--[[
October 10, 2016
Created by JoeK and distributed to the MIOS community for use.

Sept 17,2019
Maintainance changes by Majimus, long live the script

The LUA code lifx_ctrl() provides ability to control LIFX lights via Vera U15 and U17.
The implementation uses the LIFX API (RESTful) and therefore an internet connection is
required for network access to LIFX servers. The LIFX control provides a basic framework 
so can easily be modified to enhance its functionality. It's implemented as a LUA 
function and the function can be called by other LUA code or from LUUP within a 
scene on the Vera.



More info on installation and setup here:
	http://forum.micasaverde.com/index.php/topic,36961.msg275597.html#msg275597

Enjoy!  :-)
--]]

	local ltn12 = require ("ltn12")
	local https = require ("ssl.https")
	local json =  require ("lifx_ctrl_json") 
	local auth =  require ("lifx_ctrl_auth")
  
	-- changed name from json.lua to lifx_ctrl_json.json so not to conflict w/ same file name in U17
	-- using json lib from https://gist.github.com/tylerneylon/59f4bcf316be525b30ab	slightly modified for Vera				   
	
function lifx_ctrl(selector, mode, color, bright, cycles, period)
	https.TIMEOUT = 5
	--local selcolor = '"#ffffff"' -- white
	local resp = {}
	local payload = ''
	local selmethod, selurl, key, value, stat, power, connected, err
	local token = auth.getToken()

	-- Default values
	color = color or "#00ff00" -- if color nil or false then use green at brightest
	bright = bright or .2  -- if bright nil or false then use .2
	period = period or 1  -- if period nil or false then use 1
	cycles = cycles or 1  -- if period nil or false then use 1
		
	if mode == "pulse" then
		selmethod = "POST"
		selurl = "https://api.lifx.com/v1/lights/" .. selector .. "/effects/pulse"
		payload = '{"cycles": ' .. cycles .. ', "power_on": true, "persist": false, "color": "'
					.. color .. '", "period": ' .. period .. '}'
	elseif mode == "breathe" then
		selmethod = "POST"
		selurl = "https://api.lifx.com/v1/lights/" .. selector .. "/effects/breathe"
		payload = '{"cycles": ' .. cycles .. ', "power_on": true, "persist": false, "color": "'
					.. color .. '", "period": ' .. period .. '}'	
	elseif mode == "on" then
		selmethod = "PUT"
		selurl = "https://api.lifx.com/v1/lights/" .. selector .. "/state"
		payload = '{"power": "on", "color": "' .. color .. '", "brightness": ' .. bright .. '}'
	elseif mode == "off" then
		selmethod = "PUT"
		selurl = "https://api.lifx.com/v1/lights/" .. selector .. "/state"
		payload = '{"power": "off"}'
	elseif mode == "toggle" then
		selmethod = "POST"
		selurl = "https://api.lifx.com/v1/lights/" .. selector .. "/toggle"
		payload = ''
	elseif mode == "list" then
		selmethod = "GET"
		selurl = "https://api.lifx.com/v1/lights/" .. selector
		payload = ''
	elseif mode == "scene" then
		selmethod = "PUT"
		selurl = "https://api.lifx.com/v1/scenes/" .. selector .. "/activate"
		payload = '{"duration":"'.. period ..'"}'
	end
	
	local result, statuscode, response_body, status = https.request {
		url = selurl,
		method = selmethod,
		headers = {
				   ["Authorization"] = "Bearer " .. token,
				   ["Content-Type"] = "application/json",
				   ["Content-Length"] = payload:len() },
		source = ltn12.source.string(payload),
		sink = ltn12.sink.table(resp),
		protocol = "tlsv1_2"
	 }
	
	local jsondata = json.parse(resp[1])  --resp is a table where resp[1] is string containing json
	key, value = pairs(jsondata)
	if type(value["results"]) == "table" then 
		jsondata = jsondata.results[1]
	elseif type(value[1]) == "table" then 
		jsondata = jsondata[1]
	end
	for key, value in pairs(jsondata) do
		--print(key, value)
		if key == "power" then
			power = value
		elseif key == "connected" then
			connected = value
		elseif key == "status" then
			stat = value
		elseif key == "error" then
			err = value
		end
	end
	
	luup.log("lifx: " .. resp[1])

	return stat, power, connected, err, resp[1]

--[[
	local data = resp[1]
	print (string.find(data, '"status":"ok"'))

	print (data)
	print ('statuscode: ', statuscode)
	print ('result: ', result)

	print('status: ', status)

	for k, v in pairs(response_body) do
	   print(k, v)
	end
--]]

end
